package ebandal.libreoffice.comp.tests;

import org.junit.runner.RunWith;
import org.junit.runners.Suite.SuiteClasses;

import ebandal.libreoffice.comp.tests.base.UnoSuite;
import ebandal.libreoffice.comp.tests.uno.WriterTest;

@RunWith(UnoSuite.class)
@SuiteClasses({WriterTest.class})
public class UnoTests {

}
